//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//  Author: F. Poignant, floriane.poignant@gmail.com
//
// file STCyclotronDetectorMessenger.hh
//
#ifndef STCyclotronDetectorMessenger_h
#define STCyclotronDetectorMessenger_h 1

#include "globals.hh"
#include "G4UImessenger.hh"

class STCyclotronDetectorConstruction;
class G4UIdirectory;
class G4UIcmdWithADoubleAndUnit;
class G4UIcmdWithoutParameter; 
class G4UIcmdWithADouble;        
class G4UIcmdWithAString;
class G4UIcmdWithAnInteger;
class G4UIcmd;

class STCyclotronDetectorMessenger: public G4UImessenger
{
public:
  STCyclotronDetectorMessenger(STCyclotronDetectorConstruction* );
  ~STCyclotronDetectorMessenger();
    
  void SetNewValue(G4UIcommand*, G4String);
    
private:

  // Pointer to the phantom/detector 
  STCyclotronDetectorConstruction* fDet;

  G4UIdirectory *fChangeTarget, *fChangeTargetMaterial,  *fChangeFoil, *fChangeFoilMaterial;

  G4UIcmdWithoutParameter   *fUpdateMaterial, *fUpdateFoilMaterial;
  G4UIcmdWithAnInteger      *fTargetIsotopeN, *fTargetElementNComponents,*fTargetMaterialNComponents, *fFoilIsotopeN, *fFoilElementNComponents, *fFoilMaterialNComponents;
  G4UIcmdWithADouble    *fTargetIsotopeZ, *fTargetIsotopeA, *fTargetElementAbundance, *fTargetMaterialFractionMass, *fTargetMaterialNaturalElementFractionMass, *fFoilIsotopeZ, *fFoilIsotopeA, *fFoilElementAbundance, *fFoilMaterialFractionMass, *fFoilMaterialNaturalElementFractionMass, *fChangeTargetMaterialDensityCmd, *fChangeFoilMaterialDensityCmd; 
  G4UIcmdWithADoubleAndUnit *fChangeTargetThicknessCmd, *fChangeFoilThicknessCmd, *fChangeTargetDiameterCmd;
  G4UIcmdWithAString *fChangeTargetMaterialCmd, *fTargetIsotopeName, *fTargetElementName, *fTargetElementSymbole, *fTargetMaterialNaturalElement, *fFoilIsotopeName, *fFoilElementName, *fFoilElementSymbole, *fFoilMaterialNaturalElement, *fChangeFoilMaterialCmd;

};
#endif
