//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
/// \file FCALActionInitialization.cc
/// \brief Implementation of the FCALActionInitialization class

#include "FCALActionInitialization.hh"
#include "FCALPrimaryGeneratorAction.hh"
#include "FCALRunAction.hh"
#include "FCALSteppingAction.hh"
#include "FCALTBEventAction.hh"
#include "FCALSteppingVerbose.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

FCALActionInitialization::FCALActionInitialization()
 : G4VUserActionInitialization()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

FCALActionInitialization::~FCALActionInitialization()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void FCALActionInitialization::BuildForMaster() const
{
  SetUserAction(new FCALRunAction);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void FCALActionInitialization::Build() const
{
  SetUserAction(new FCALPrimaryGeneratorAction);
  SetUserAction(new FCALRunAction);
    FCALSteppingAction* sa = new FCALSteppingAction;
    SetUserAction(sa);
    SetUserAction(new FCALTBEventAction(sa));
  //SetUserAction(new FCALEventAction);
}  

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
G4VSteppingVerbose* FCALActionInitialization::InitializeSteppingVerbose() const
{
    return new FCALSteppingVerbose;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

