//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file eventgenerator/particleGun/include/PrimaryGeneratorAction.hh
/// \brief Definition of the PrimaryGeneratorAction class
//
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#ifndef PrimaryGeneratorAction_h
#define PrimaryGeneratorAction_h 1

#include "G4VUserPrimaryGeneratorAction.hh"
#include "globals.hh"

class PrimaryGeneratorAction0;
class PrimaryGeneratorAction1;
class PrimaryGeneratorAction2;
class PrimaryGeneratorAction3;
class PrimaryGeneratorAction4;
class PrimaryGeneratorMessenger;

class G4ParticleGun;
class G4Event;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class PrimaryGeneratorAction : public G4VUserPrimaryGeneratorAction
{
  public:
    PrimaryGeneratorAction();
    ~PrimaryGeneratorAction() override;

  public:
    void GeneratePrimaries(G4Event*) override;

  public:
    G4ParticleGun* GetParticleGun() { return fParticleGun; };

    void SelectAction(G4int i) { fSelectedAction = i; };
    G4int GetSelectedAction() { return fSelectedAction; };

    PrimaryGeneratorAction0* GetAction0() { return fAction0; };
    PrimaryGeneratorAction1* GetAction1() { return fAction1; };
    PrimaryGeneratorAction2* GetAction2() { return fAction2; };
    PrimaryGeneratorAction3* GetAction3() { return fAction3; };
    PrimaryGeneratorAction4* GetAction4() { return fAction4; };

  private:
    G4ParticleGun* fParticleGun = nullptr;

    PrimaryGeneratorAction0* fAction0 = nullptr;
    PrimaryGeneratorAction1* fAction1 = nullptr;
    PrimaryGeneratorAction2* fAction2 = nullptr;
    PrimaryGeneratorAction3* fAction3 = nullptr;
    PrimaryGeneratorAction4* fAction4 = nullptr;
    G4int fSelectedAction = 0;

    PrimaryGeneratorMessenger* fGunMessenger = nullptr;
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#endif
