//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
//
//
// Class Description:
//
// This class manages the Bremsstrahlung process for hadrons
// it inherites from G4VContinuousDiscreteProcess via G4VEnergyLossProcess.
//

// -------------------------------------------------------------------
//

#ifndef G4LDMBremsstrahlung_h
#define G4LDMBremsstrahlung_h 1

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4MuBremsstrahlung.hh"
#include "globals.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

class G4LDMBremsstrahlung : public G4MuBremsstrahlung

{
  public:
    explicit G4LDMBremsstrahlung(const G4String& processName = "ldmBrems");

    virtual ~G4LDMBremsstrahlung();

    virtual G4bool IsApplicable(const G4ParticleDefinition& p) override;

    virtual G4double MinPrimaryEnergy(const G4ParticleDefinition*, const G4Material*,
                                      G4double) override;

  protected:
    virtual void InitialiseEnergyLossProcess(const G4ParticleDefinition*,
                                             const G4ParticleDefinition*) override;

  private:
    G4LDMBremsstrahlung& operator=(const G4LDMBremsstrahlung& right) = delete;
    G4LDMBremsstrahlung(const G4LDMBremsstrahlung&) = delete;
};

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

#endif
