//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file hadronic/Hadr01/src/RunAction.cc
/// \brief Implementation of the RunAction class
//
//
// -------------------------------------------------------------
//
//    GEANT4 class file
//    RunAction
//
// -------------------------------------------------------------

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "RunAction.hh"

#include "HistoManager.hh"

#include "G4CascadeChannelTables.hh"
#include "G4NuclearLevelData.hh"
#include "G4Timer.hh"
#include "G4UImanager.hh"
#include "G4VVisManager.hh"

#include <iosfwd>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

RunAction::RunAction() : G4UserRunAction()
{
  fTimer = new G4Timer();
  fTimer->Start();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void RunAction::BeginOfRunAction(const G4Run* aRun)
{
  G4int id = aRun->GetRunID();
  G4cout << "### Run " << id << " start" << G4endl;
  fTimer->Stop();
  G4cout << "Initialisation time:  " << *fTimer << G4endl;

  delete fTimer;
  fTimer = new G4Timer();
  fTimer->Start();

  (HistoManager::GetPointer())->BeginOfRun();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void RunAction::EndOfRunAction(const G4Run*)
{
  fTimer->Stop();
  G4cout << "RunAction::EndOfRunAction:  " << *fTimer << G4endl;
  G4cout << "RunAction: End of run actions are started" << G4endl;
  auto man = HistoManager::GetPointer();
  man->EndOfRun();
  G4int key = man->PrintBertiniXS();
  if (0 == key) {
    G4cout << "RunAction: Print all Bertini cross sections" << G4endl;
    G4CascadeChannelTables::Print();
  }
  else if (0 < key) {
    G4cout << "RunAction: Print Bertini cross section key=" << key << G4endl;
    G4CascadeChannelTables::PrintTable(key, G4cout);
  }
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
