//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file persistency/gdml/G02/include/G02ChamberParameterisation.hh
/// \brief Definition of the G02ChamberParameterisation class
//
//
//
// Class G02ChamberParameterisation
//
// A parameterisation that describes a series of boxes along Z
// The boxes have equal width; their lengths are a linear equation.
// They are spaced an equal distance apart, starting from a given location.
//
// ----------------------------------------------------------------------------

#ifndef G02ChamberParameterisation_H
#define G02ChamberParameterisation_H 1

#include "G4VPVParameterisation.hh"
#include "globals.hh"

class G4VPhysicalVolume;
class G4Box;

// Dummy declarations to get rid of warnings ...
//
class G4Trd;
class G4Trap;
class G4Cons;
class G4Orb;
class G4Sphere;
class G4Ellipsoid;
class G4Torus;
class G4Para;
class G4Hype;
class G4Tubs;
class G4Polycone;
class G4Polyhedra;

// ----------------------------------------------------------------------------

/// Chamber parametrisation used in the GDML read/write example

class G02ChamberParameterisation : public G4VPVParameterisation
{
  public:
    G02ChamberParameterisation(G4int NoChambers, G4double startZ, G4double spacing,
                               G4double widthChamber, G4double lengthInitial, G4double lengthFinal);

    ~G02ChamberParameterisation();

    void ComputeTransformation(const G4int copyNo, G4VPhysicalVolume* physVol) const;

    void ComputeDimensions(G4Box& trackerLayer, const G4int copyNo,
                           const G4VPhysicalVolume* physVol) const;

  private:  // Dummy declarations to get rid of warnings ...
    void ComputeDimensions(G4Trd&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Trap&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Cons&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Sphere&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Orb&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Ellipsoid&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Torus&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Para&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Hype&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Tubs&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Polycone&, const G4int, const G4VPhysicalVolume*) const {}
    void ComputeDimensions(G4Polyhedra&, const G4int, const G4VPhysicalVolume*) const {}

  private:
    G4int fNoChambers;
    G4double fStartZ;
    G4double fHalfWidth;  //  The half-width of each tracker chamber
    G4double fSpacing;  //  The distance between the chambers' center
    G4double fHalfLengthFirst;  //  The first half-length
    G4double fHalfLengthIncr;  //  The Increment for the half-length
};

// ----------------------------------------------------------------------------

#endif
