//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// G4VStateDependent class implementation
//
// Authors: G.Cosmo, M.Asai - November 1996
// --------------------------------------------------------------------

#include "G4VStateDependent.hh"
#include "G4StateManager.hh"

G4VStateDependent::G4VStateDependent(G4bool bottom)
{
  G4StateManager* stateManager = G4StateManager::GetStateManager();
  stateManager->RegisterDependent(this, bottom);
}

G4VStateDependent::~G4VStateDependent()
{
  G4StateManager* stateManager = G4StateManager::GetStateManager();
  stateManager->DeregisterDependent(this);
}

G4VStateDependent::G4VStateDependent(const G4VStateDependent& right)
{
  *this = right;
}

G4VStateDependent& G4VStateDependent::operator=(const G4VStateDependent& right)
{
  if(&right == this)
  {
    return *this;
  }
  *this = right;
  return *this;
}

G4bool G4VStateDependent::operator==(const G4VStateDependent& right) const
{
  return (this == &right);
}

G4bool G4VStateDependent::operator!=(const G4VStateDependent& right) const
{
  return (this != &right);
}
