//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// GEANT4 Class header file
//
// File name:  G4OrePowellAtRestModel
//
// Author:     I.Semeniouk & D.Bernard
//
// Creation date:  4 Juin 2024
//
// Class Description:
//
// Simple AtRest positron 3-gamma annihilation model.
// Electron of media is assumed to have zero kinetic energy.
// Ortho - Positronium three-photon annihilation by Ore and
// Powell, Phys. Rev. 75 (1949).
// Polarisation set randomly.
//
// -------------------------------------------------------------------
//

#ifndef G4OrePowellAtRestModel_h
#define G4OrePowellAtRestModel_h 1

#include "G4VPositronAtRestModel.hh"

class G4OrePowellAtRestModel : public G4VPositronAtRestModel
{

public:

  G4OrePowellAtRestModel();

  ~G4OrePowellAtRestModel() override = default;

  void SampleSecondaries(std::vector<G4DynamicParticle*>& secParticles,
			 G4double&, const G4Material*) const override;

  void PrintGeneratorInformation() const override;

  G4OrePowellAtRestModel& operator=
  (const  G4OrePowellAtRestModel& right) = delete;
  G4OrePowellAtRestModel(const G4OrePowellAtRestModel&) = delete;
};

#endif
