//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// INCL++ intra-nuclear cascade model
// Alain Boudard, CEA-Saclay, France
// Joseph Cugnon, University of Liege, Belgium
// Jean-Christophe David, CEA-Saclay, France
// Pekka Kaitaniemi, CEA-Saclay, France, and Helsinki Institute of Physics, Finland
// Sylvie Leray, CEA-Saclay, France
// Davide Mancusi, CEA-Saclay, France
//
#define INCLXX_IN_GEANT4_MODE 1

#include "globals.hh"

/** \file G4INCLIPhaseSpaceGenerator.hh
 * \brief Abstract interface for the phase-space generators
 *
 * \date 15th November 2013
 * \author Davide Mancusi
 */

#ifndef G4INCLIPHASESPACEGENERATOR_HH
#define G4INCLIPHASESPACEGENERATOR_HH

#include "G4INCLParticle.hh"
#include "G4INCLThreeVector.hh"

namespace G4INCL {
  /// \brief Abstract interface for the phase-space generators
  class IPhaseSpaceGenerator {
    public:

      IPhaseSpaceGenerator() {}
      virtual ~IPhaseSpaceGenerator() {}

      /// \brief Generate an event in the CM frame
      virtual void generate(const G4double sqrtS, ParticleList &particles) = 0;

  };
}

#endif
