//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// G4VUserPrimaryGeneratorAction
//
// Class description:
//
// Abstract base class for user's mandatory action class for primary
// vertex/particle generation. This class has only one pure virtual
// method GeneratePrimaries() which is invoked from G4RunManager
// during the event loop.
// This class is NOT intended for generating primary vertex/particle
// by itself. This class should:
// - have one or more G4VPrimaryGenerator concrete classes such as G4ParticleGun
// - set/change properties of generator(s)
// - pass G4Event object so that the generator(s) can generate primaries.

// Original author: M.Asai, 1999
// --------------------------------------------------------------------
#ifndef G4VUserPrimaryGeneratorAction_hh
#define G4VUserPrimaryGeneratorAction_hh 1

class G4Event;

class G4VUserPrimaryGeneratorAction
{
  public:
    G4VUserPrimaryGeneratorAction();
    virtual ~G4VUserPrimaryGeneratorAction() = default;

    virtual void GeneratePrimaries(G4Event* anEvent) = 0;
};

#endif
