//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// This example is provided by the Geant4-DNA collaboration
// Any report or published results obtained using the Geant4-DNA software
// shall cite the following Geant4-DNA collaboration publication:
// Med. Phys. 37 (2010) 4692-4708
// J. Comput. Phys. 274 (2014) 841-882
// The Geant4-DNA web site is available at http://geant4-dna.org
//
// $Id$
//
/// \file TimeStepAction.hh
/// \brief Definition of the TimeStepAction class

#ifndef ITACTION_H
#define ITACTION_H

#include "G4MolecularConfiguration.hh"
#include "G4MoleculeIterator.hh"
#include "G4MoleculeTable.hh"
#include "G4UserTimeStepAction.hh"

class DetectorConstruction;
class G4Molecule;

class TimeStepAction : public G4UserTimeStepAction
{
  public:
    TimeStepAction();
    virtual ~TimeStepAction();
    TimeStepAction(const TimeStepAction& other);
    TimeStepAction& operator=(const TimeStepAction& other);

    virtual void StartProcessing() { ; }

    virtual void UserPreTimeStepAction();
    virtual void UserPostTimeStepAction() { ; }

    virtual void UserReactionAction(const G4Track& /*trackA*/, const G4Track& /*trackB*/,
                                    const std::vector<G4Track*>* /*products*/);

    virtual void EndProcessing() { ; }

    void Save(G4MolecularConfiguration* molconf);
    void SaveMoleculeInfo(G4Track* track, G4int molID, const G4String&
                          /*moleculeName*/);

  private:
    const DetectorConstruction* fpDetector;
};

#endif  // ITACTION_H
